﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace AGs_em_Data_Mining
{
    public partial class frmPrincipal : Form
    {
        public frmPrincipal()
        {
            InitializeComponent();
        }

        private void frmPrincipal_Load(object sender, EventArgs e)
        {
            cargaClasse(@"..\..\defaultClasse.ag");
            cargaAtributo(@"..\..\defaultAtributo.ag");

            if (lbxAtributo.Items.Count > 0)
            {
                cargaBaseDados(@"..\..\defaultDataBase.ag");
            }
        }

        #region Carga
        private void cargaClasse(String caminho)
        {
            String linha;
            Int32 count = 0;

            try
            {
                System.IO.StreamReader file = new System.IO.StreamReader(caminho);

                lbxClasse.Items.Clear();

                while ((linha = file.ReadLine()) != null && ++count <= 50)
                {
                    lbxClasse.Items.Add(linha);
                }

                if (lbxClasse.Items.Count > 0)
                {
                    lbxClasse.SelectedIndex = 0;
                }

                if (count > 50)
                {
                    txtErroCargaClasseAtributo.Text += "Número máximo de classes é 50. Não foram inseridos todas classes do arquivo.\r\n";
                }
            }
            catch (Exception e)
            {
                txtErroCargaClasseAtributo.Text += e.Message + "\r\n";
            }
        }

        private void cargaAtributo(String caminho)
        {
            String linha;
            Int32 count = 0;

            try
            {
                System.IO.StreamReader file = new System.IO.StreamReader(caminho);

                lbxAtributo.Items.Clear();

                while ((linha = file.ReadLine()) != null && ++count <= 100)
                {
                    lbxAtributo.Items.Add(linha);
                }

                if (lbxAtributo.Items.Count > 0)
                {
                    lbxAtributo.SelectedIndex = 0;
                }

                if (count > 100)
                {
                    txtErroCargaClasseAtributo.Text += "Número máximo de atributos é 100. Não foram inseridos todos atributos do arquivo.\r\n";
                    dgvCargaBaseDado.ColumnCount = count;
                }
                else
                {
                    dgvCargaBaseDado.ColumnCount = count + 1;
                }

                DataGridViewCellStyle style = dgvCargaBaseDado.ColumnHeadersDefaultCellStyle;
                style.Font = new Font(dgvCargaBaseDado.Font, FontStyle.Bold);

                dgvCargaBaseDado.EditMode = DataGridViewEditMode.EditOnF2;
                dgvCargaBaseDado.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.DisplayedCellsExceptHeaders;
                dgvCargaBaseDado.ColumnHeadersBorderStyle = DataGridViewHeaderBorderStyle.Raised;
                dgvCargaBaseDado.CellBorderStyle = DataGridViewCellBorderStyle.Single;
                dgvCargaBaseDado.GridColor = SystemColors.ActiveBorder;
                dgvCargaBaseDado.RowHeadersVisible = false;

                for (Int32 i = 0; i < lbxAtributo.Items.Count; i++)
                {
                    if (lbxAtributo.Items[i].ToString().Contains("|"))
                    {
                        dgvCargaBaseDado.Columns[i].Name = lbxAtributo.Items[i].ToString().Substring(0, lbxAtributo.Items[i].ToString().IndexOf('|'));
                    }
                    else
                    {
                        dgvCargaBaseDado.Columns[i].Name = lbxAtributo.Items[i].ToString();
                    }
                }

                dgvCargaBaseDado.Columns[lbxAtributo.Items.Count].Name = ((Int32)(lbxAtributo.Items.Count + 1)).ToString() + ": Classe";
            }
            catch (Exception e)
            {
                txtErroCargaClasseAtributo.Text += e.Message + "\r\n";
            }
        }

        private void cargaBaseDados(String caminho)
        {
            String linha, tmp;
            Int32 count;
            bool inserir;

            try
            {
                String[] str = new String[dgvCargaBaseDado.ColumnCount];
                System.IO.StreamReader file = new System.IO.StreamReader(caminho);
                DataGridViewRowCollection rows = dgvCargaBaseDado.Rows;

                while ((linha = file.ReadLine()) != null)
                {
                    tmp = "";
                    count = 0;
                    inserir = true;

                    foreach (Char caracter in linha)
                    {
                        if (caracter.Equals(',') && count < str.Length)
                        {
                            try
                            {
                                Convert.ToInt32(tmp);
                                str[count++] = tmp;
                                tmp = "";
                            }
                            catch (Exception e)
                            {
                                txtErroCargaClasseAtributo.Text += e.Message + "\r\n" + linha + "\r\n";
                                inserir = false;
                                break;
                            }
                        }
                        else
                        {
                            if (tmp.Length < 8)
                            {
                                tmp += caracter;
                            }
                        }
                    }

                    if (inserir)
                    {
                        if (!str.Equals("") && count < str.Length)
                        {
                            str[count++] = tmp;
                        }

                        rows.Add(str);
                    }
                }
            }
            catch (Exception e)
            {
                txtErroCargaClasseAtributo.Text += e.Message + "\r\n";
            }
        }
        #endregion

        #region Ações dos Botões de Carga
        private void lbxClasse_DoubleClick(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();

            openFileDialog1.Title = "Carga das Classes";
            openFileDialog1.Filter = "Arquivo de Classes (*.ag)|*.ag|All files (*.*)|*.*";
            openFileDialog1.RestoreDirectory = true;

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                Stream myStream = openFileDialog1.OpenFile();

                if (myStream != null)
                {
                    cargaClasse(((FileStream)myStream).Name);
                    myStream.Close();
                }
            }
        }

        private void lbxAtributo_DoubleClick(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();

            openFileDialog1.Title = "Carga dos Atributos";
            openFileDialog1.Filter = "Arquivo de Atributos (*.ag)|*.ag|All files (*.*)|*.*";
            openFileDialog1.RestoreDirectory = true;

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                Stream myStream = openFileDialog1.OpenFile();

                if (myStream != null)
                {
                    cargaAtributo(((FileStream)myStream).Name);
                    myStream.Close();
                }
            }
        }

        private void btnCargaDado_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();

            openFileDialog1.Title = "Carga da Base de Dados";
            openFileDialog1.Filter = "Base de Dados (*.ag)|*.ag|All files (*.*)|*.*";
            openFileDialog1.RestoreDirectory = true;

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                Stream myStream = openFileDialog1.OpenFile();

                if (myStream != null)
                {
                    cargaBaseDados(((FileStream)myStream).Name);
                    myStream.Close();
                }
            }
        }

        private void btnLimparBaseDado_Click(object sender, EventArgs e)
        {
            dgvCargaBaseDado.Rows.Clear();
        }
        #endregion
        
        #region Processamento do AG
        private void btnAGDataMining_Click(object sender, EventArgs e)
        {
            Mundo mundo = new Mundo(Convert.ToInt32(txtTamanhoPopulacao.Text),
                                    Convert.ToInt32(txtTaxaCrossover.Text),
                                    Convert.ToInt32(txtTaxaMutacao.Text),
                                    Convert.ToInt32(txtNumeroGeracao.Text),
                                    dgvCargaBaseDado.ColumnCount - 1,
                                    Convert.ToInt32(txtLimiar.Text),
                                    lbxClasse.SelectedIndex + 1,
                                    rdbElitista.Checked ? 0 : 1,
                                    rdbEstocastico.Checked ? 0 : (rdbTorneioSimples.Checked ? 1 : 2));
            
            Individuo melhorIndividuo;

            List<Int32[]> baseDados = BaseDados();

            melhorIndividuo = mundo.AGDataMining(baseDados);

            if (melhorIndividuo != null)
            {
                txtErroCargaClasseAtributo.Text += "BASE DE TREINAMENTO\r\n";
                txtErroCargaClasseAtributo.Text += "Aptidão: " + melhorIndividuo.aptidao.ToString() + "\r\n";
                txtErroCargaClasseAtributo.Text += "Se: " + melhorIndividuo.sensitividade.ToString() + "\r\n";
                txtErroCargaClasseAtributo.Text += "Sp: " + melhorIndividuo.especificidade.ToString() + "\r\n";

                mundo.TestaMelhorIndividuo(baseDados, melhorIndividuo);

                txtErroCargaClasseAtributo.Text += "BASE DE TESTE\r\n";
                txtErroCargaClasseAtributo.Text += "Aptidão: " + melhorIndividuo.aptidao.ToString() + "\r\n";
                txtErroCargaClasseAtributo.Text += "Se: " + melhorIndividuo.sensitividade.ToString() + "\r\n";
                txtErroCargaClasseAtributo.Text += "Sp: " + melhorIndividuo.especificidade.ToString() + "\r\n";
                txtErroCargaClasseAtributo.Text += "REGRA:\r\n";

                for (Int32 i = 0; i < melhorIndividuo.cromossomo.Count; i++)
                {
                    if (melhorIndividuo.cromossomo[i].Weight >= Convert.ToInt32(txtLimiar.Text))
                    {
                        string concat;

                        if (0 == melhorIndividuo.cromossomo[i].Operator)
                        {
                            concat = "=";
                        }
                        else if (1 == melhorIndividuo.cromossomo[i].Operator)
                        {
                            concat = "!=";
                        }
                        else if (2 == melhorIndividuo.cromossomo[i].Operator)
                        {
                            concat = ">=";
                        }
                        else
                        {
                            concat = "<";
                        }

                        txtErroCargaClasseAtributo.Text += "IF " + lbxAtributo.Items[i].ToString() + " " + concat + " " + melhorIndividuo.cromossomo[i].Value + "\r\n";
                    }
                }
            }
        }

        private List<Int32[]> BaseDados()
        {
            List<Int32[]> baseDados = new List<Int32[]>(dgvCargaBaseDado.RowCount - 1);
            Int32[] linha;
            Int32 i, j;

            for (i = 0; i < baseDados.Capacity; i++)
            {
                linha = new Int32[dgvCargaBaseDado.ColumnCount];

                for (j = 0; j < linha.Length; j++)
                {
                    linha[j] = Convert.ToInt32(dgvCargaBaseDado.Rows[i].Cells[j].EditedFormattedValue);
                }

                baseDados.Add(linha);
            }

            return baseDados;
        }
        #endregion
     }
}